var LOGFILE_NAME = "remote-connectivity.log";
var lInfo = new Logger(LOGFILE_NAME);
var lWarn = new Logger(LOGFILE_NAME);
lInfo.logln("event raised = " + raisedEvent.name);

/**
 * store temporary relay link to property tree node tempAccess/relayLink
 * @param relayLink
 */
function saveTempRelayLink(relayLink) {
    try {
        if ((relayLink != null) && (relayLink != undefined)) {
            lInfo.logln("saveTempRelayLink: " + relayLink);
            Property.load();
            lInfo.logln("Save tempLink in property tree: " + relayLink);
            Property.setProperty('tempAccess/relayLink', relayLink);
            Property.setFlag('tempAccess/relayLink', 'ARCHIVE', true);
            Property.store();    
        } else {
            lWarn.logln("cannot saveTempRelayLink");
        }
    }
    catch(e) {
        lWarn.logln(e);
    }
}

/**
 * store expiration time of temporary relay link to property tree node tempAccess/expirationTime
 * @param expirationTime
 */
function saveExpirationTime(expirationTime) {
    try {
        if ((expirationTime != null) && (expirationTime != undefined)) {
            lInfo.logln("saveExpirationTime: " + expirationTime);
            Property.load();
            lInfo.logln("Save expirationTime in property tree: " + expirationTime);
            Property.setProperty('tempAccess/expirationTime', expirationTime);
            Property.setFlag('tempAccess/expirationTime', 'ARCHIVE', true);
            Property.store();  
        } else {
            lWarn.logln("cannot saveExpirationTime");
        }
    }
    catch(e) {
        lWarn.logln(e);
    }
}

/**
 * calculate the expiration time of the templink later on this information should be provided by cloud.
 * the expiration time is return as string in curent dSS time.
 * @ startTime  start time in UTC for validity period
 * @ validity validity of temp link in hours
 */
function calculateExpirationTime(startTime, validity) {
    try {
        var date =  new Date();
        var UTCOffsetInMinutes = date.getTimezoneOffset();
        lInfo.logln("calculateExpirationTime UTCOffset: " + UTCOffsetInMinutes/60);        
        if ((startTime != undefined) && (validity != undefined)) {
            var expirationDate = new Date(startTime);
            var expirationTime = expirationDate.getTime() + (validity*60*60*1000);
            //convert to time zone of dss
            expirationTime = expirationTime - UTCOffsetInMinutes*60*1000;    
            expirationDate.setTime(expirationTime);
            return expirationDate.toLocaleString();
        } else {
            return '';
        }
    }
    catch(e) {
        lWarn.logln(e);
        return '';
    }
}

/**
 * Main function, called when script is invoked for event handling.
 *Tthe called event is stored in global variable raisedEvent
 */
function handleEvent() {
    try {
        if (raisedEvent.name === 'remote-connectivity-disabled') {
            lInfo.logln("remote connectivity disabled event");
            if ((raisedEvent.parameter.success == 'true') || (('' + raisedEvent.parameter.success) == '0')) {
                if (raisedEvent.parameter.success == 'true') {
                    (new Event("remote-connectivity.answer",{actions:'DeactivateRemoteConnectivity',result:true})).raise();
                } else {
                    (new Event("remote-connectivity.answer",{actions:'DeactivateRemoteConnectivity',result:false, code: raisedEvent.parameter.code, servicecode: raisedEvent.parameter.servicecode})).raise();
                }
                //now it is ok to uninstall this addon
                Property.setProperty('system/preventUninstall',false);
                Property.setFlag('system/preventUninstall', 'ARCHIVE', true);
                Property.store();           
            } else {
                (new Event("remote-connectivity.answer",{actions:'DeactivateRemoteConnectivity',result:false})).raise();
            }    
        }
        
        if (raisedEvent.name === 'remote-connectivity') { 
            if (raisedEvent.parameter.success != null) {
                if ((raisedEvent.parameter.mode) && (raisedEvent.parameter.mode =='temp')) {
                    lInfo.logln('get event for temp link activation');
                    if ((raisedEvent.parameter.success == 'true') || (('' + raisedEvent.parameter.success) == '0')) {
                        if (raisedEvent.parameter.relaylink!=null){
                            lInfo.logln('relayLink: ' + raisedEvent.parameter.relaylink);
                            saveTempRelayLink(raisedEvent.parameter.relaylink);
                            (new Event("remote-connectivity.answer",{actions:'ActivateTempLink',result:true, relaylink:raisedEvent.parameter.relaylink})).raise();
                        } else {
                            lWarn.logln('temp link activation internal failure');
                            (new Event("remote-connectivity.answer",{actions:'ActivateTempLink',result:false})).raise();                    
                        }
                    } else { 
                        lWarn.logln('temp link activation failed, code: ' + raisedEvent.parameter.code + ' servicecode: ' + raisedEvent.parameter.servicecode);
                        (new Event("remote-connectivity.answer",{actions:'ActivateTempLink',result:false, code: raisedEvent.parameter.code, servicecode: raisedEvent.parameter.servicecode})).raise();
                    }
                } else {
                    lInfo.logln('get event for permanent link activation done');
                    if (raisedEvent.parameter.success == 'true') {
                    	if (raisedEvent.parameter.osptoken!=null) {
                    	    lInfo.logln('-- token ' + raisedEvent.parameter.osptoken);
                    		if (raisedEvent.parameter.relaylink!=null){
                    			Property.load();
                    			lInfo.logln('save token');
                    		    Property.setProperty('OSPToken', raisedEvent.parameter.osptoken);
                    		    Property.setFlag('OSPToken', 'ARCHIVE', true);
                    		    //as long as remote connectivity is enabled this addon may not be uninstalled
                    		    Property.setProperty('system/preventUninstall',true);
                    	        Property.setFlag('system/preventUninstall', 'ARCHIVE', true);
                    		    Property.store();			
                    		    (new Event("remote-connectivity.answer",{actions:'ActivateRemoteConnectivity',result:true})).raise();
                    		}
                    	}
                    } else { 
                        lWarn.logln('permanent link activation failed, code: ' + raisedEvent.parameter.code + ' servicecode: ' + raisedEvent.parameter.servicecode);
                        (new Event("remote-connectivity.answer",{actions:'ActivateRemoteConnectivity',result:false, code: raisedEvent.parameter.code, servicecode: raisedEvent.parameter.servicecode})).raise();
                    } 
                }
            }
        
            if (raisedEvent.parameter.actions!=null) {
                lInfo.logln('action: ' + raisedEvent.parameter.actions);
            	Property.load();
            	switch (raisedEvent.parameter.actions) {
            		case 'CheckUserByMAC': {
            			var result=(RemoteConnectivity.CheckUserByMAC());
            			if (result!=null) {
        					if (result.ReturnCode==0) {
                                (new Event("remote-connectivity.answer",{actions:'CheckUserByMAC',result:true})).raise();
                                // make a apartment model change
                                var sName=Property.getProperty('/apartment/zones/zone0/name');
                                Property.setProperty('/apartment/zones/zone0/name', 'temporary');
                                Property.setProperty('/apartment/zones/zone0/name', sName);
        					} else {
                                (new Event("remote-connectivity.answer",{actions:'CheckUserByMAC',result:false})).raise();
        					}
            			} else {
            			    lWarn.logln('CheckUserByMAC call failed');
            				(new Event("remote-connectivity.answer",{actions:'CheckUserByMAC',result:false})).raise();
            			}
            		}
            		break;
            		case 'CheckExistingTempdSConfiguratorLink': {
            		    var result=(RemoteConnectivity.CheckExistingTempdSConfiguratorLink());
            		    if (result!=null) {
            		        lInfo.logln('CheckExistingTempdSConfiguratorLink');
            		        if (result.ReturnCode == 0) {
                                if (result.Response!=null) {
                                    if ((result.Response.TempdSConfoguratorLinkExists === 'true') ||  (result.Response.TempdSConfoguratorLinkExists === true)) {
                                        var expirationTime = calculateExpirationTime(result.Response.ValidFrom, result.Response.TempLinkValidity);
                                        saveExpirationTime(expirationTime);
                                        lInfo.logln('--link exists: ValidFrom ' + result.Response.ValidFrom +" TempLinkValidity " + result.Response.TempLinkValidity + " ExpirationTime " + expirationTime);
                                        (new Event("remote-connectivity.answer",{actions:'CheckExistingTempdSConfiguratorLink', result:true, tempLinkExist:true, expirationTime: expirationTime, tempLinkValidity:result.Response.TempLinkValidity })).raise();
                                    } else {
                                        Property.load();
                                        var baseNode = Property.getNode('tempAccess');
                                        if (baseNode) {
                                            baseNode.removeChild('relayLink');
                                            baseNode.removeChild('expirationTime');
                                            Property.store();                      
                                        }
                                        lInfo.logln('--link does not exists, we send event');
                                        (new Event("remote-connectivity.answer",{actions:'CheckExistingTempdSConfiguratorLink', result:true, tempLinkExist:false})).raise();
                                    }
                                } else {
                                    lWarn.logln('--Response == null');
                                    (new Event("remote-connectivity.answer",{actions:'CheckExistingTempdSConfiguratorLink',result:false})).raise();
                                }
            		        } else {
            		            lWarn.logln('--ReturnCode != 0');
                                (new Event("remote-connectivity.answer",{actions:'CheckExistingTempdSConfiguratorLink',result:false})).raise();    		            
            		        }
                        } else {
                            lWarn.logln('--ReturnCode != 0');
                            (new Event("remote-connectivity.answer",{actions:'CheckExistingTempdSConfiguratorLink',result:false})).raise();
                        }
            		}
            		break;
                    case 'EnableToken': {
                        lInfo.logln('EnableToken');
                        if (raisedEvent.parameter.token!=null) {
                            if (Property.getNode('/system/security/applicationTokens/pending/'+raisedEvent.parameter.token)!=null) {
                                if (raisedEvent.parameter.osptoken!=null) {
                                    if (Property.getNode('OSPToken')!=null) {
                                        if (Property.getNode('OSPToken').getValue()==raisedEvent.parameter.osptoken){
                                            lInfo.logln('start enableToken');
                                            RemoteConnectivity.enableToken(raisedEvent.parameter.token);
                                            lInfo.logln('ok ' + oToken);
                                            (new Event("remote-connectivity.answer",{actions:'EnableToken',token:raisedEvent.parameter.token,resultcode:0})).raise();
                                        } else {
                                            lInfo.logln('wrong osp-token');
                                            (new Event("remote-connectivity.answer",{actions:'EnableToken',resultcode:-4,error:'wrong OSP Token'})).raise();
                                        }
                                    } else {
                                        lInfo.logln('no Connectivity in dSS enabled');
                                        (new Event("remote-connectivity.answer",{actions:'EnableToken',resultcode:-1,error:'no Connectivity in dSS enabled'})).raise();
                                    }
                                } else {
                                    lInfo.logln('no OSP-Token delivered');
                                    (new Event("remote-connectivity.answer",{actions:'EnableToken',resultcode:-2,error:'no OSP-Token delivered'})).raise();                 
                                }
                            } else {
                                lInfo.logln('token not present');
                                (new Event("remote-connectivity.answer",{actions:'EnableToken',resultcode:-4,error:'token to enable not present'})).raise();                                    
                            }
                        } else {
                            lInfo.logln('no App-Name delivered');
                            (new Event("remote-connectivity.answer",{actions:'EnableToken',resultcode:-3,error:'no token delivered'})).raise();                                 
                        }
                    }
                    break;
            		case 'saveUserName' : {
            		    Property.load();
            		    var userName = unescape(raisedEvent.parameter.userName);
                        lInfo.logln('save user name "' + userName + '" for later use');
                        Property.setProperty('username', userName);
                        Property.setFlag('username', 'ARCHIVE', true);
                        Property.store();           
            		}
            		break;
            		case 'cleanUpPropertyTree' : {
            		    Property.load();
            		    if (Property.getNode('Relaylink')!=null) {
            		        Property.getNode('/scripts/remote-connectivity').removeChild('Relaylink');
            		    }
            		    Property.store();
            		}
            		break;
            		default: {
            		    lWarn.logln('unexpected event action: ' + raisedEvent.parameter.actions);
            		}
            	}
            }
        } //name = remote connectivity
    } //end try
    catch(e) {
        lWarn.logln('exception processing event');
    }
} //end function

handleEvent();